/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: afunix.c,v 1.4 1999/03/15 13:12:30 bazsi Exp $
 *
 ***************************************************************************/

#include "afunix.h"
#include "format.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <unistd.h>
#include <errno.h>

#include "afunix.c.x"

int open_unix_socket(struct sockaddr_un *sun, int proto, int *sockfd)
{
	int socktype = (proto == AFUNIX_PROTO_DGRAM ? SOCK_DGRAM : SOCK_STREAM);
	int fd;
	
	fd = socket(AF_UNIX, socktype, 0);
	if (fd != -1) {
		unlink(sun->sun_path);
		errno = 0;
		if (bind(fd, (struct sockaddr *) sun, sizeof(struct sockaddr_un)) == -1) {
			werror("cannot bind af_unix socket to %s (%s)\n", sun->sun_path, strerror(errno));
			close(fd);
			return 0;
		}
		chmod(sun->sun_path, 0666);
		*sockfd = fd;
		return 1;
	}
	/* indicate failure */
	return 0;
}


/* CLASS:
     (class
       (name afunix_source_connection)
       (super log_handler_pipe)
       (vars
         (backend object io_backend)
         (fd simple int)
         (src object io_fd)
         (next_connection object afunix_source_connection)))
*/

static int do_init_afunix_source_connection(struct log_handler *c)
{
	CAST(afunix_source_connection, self, c);
	
	self->src = io_read(self->backend, self->fd, 
			make_read_line(make_log_reader(self->super.next)),
			NULL);
	return 0;
}

struct afunix_source_connection *make_afunix_source_connection(struct io_backend *backend, int fd, struct log_handler *next)
{
	NEW(afunix_source_connection, self);

	self->super.super.init = do_init_afunix_source_connection;
	self->super.next = next;
	self->backend = backend;
	self->fd = fd;
	return self;
}

/* CLASS:
     (class
       (name afunix_accept_callback)
       (super fd_callback)
       (vars
         (backend object io_backend)
         (connections pointer (object afunix_source_connection))
         (next object log_handler)))
*/

static int do_accept_unix_connection(struct fd_callback **c, int fd)
{
	CAST(afunix_accept_callback, self, *c);
	struct afunix_source_connection *conn;
	
	conn = make_afunix_source_connection(self->backend, fd, self->next);
	conn->next_connection = *(self->connections);
	*(self->connections) = conn;
	return INIT_LOG_HANDLER(conn);
}

struct fd_callback *make_afunix_accept_callback(struct io_backend *backend, struct afunix_source_connection **connections, struct log_handler *next)
{
	NEW(afunix_accept_callback, self);
	
	self->super.f = do_accept_unix_connection;
	self->backend = backend;
	self->connections = connections;
	self->next = next;
	return &self->super;
}

/* CLASS:
     (class
       (name afunix_source)
       (super log_source_driver)
       (vars
         (backend object io_backend)
         (src object io_fd)
         (listen object listen_fd)
         (connections object afunix_source_connection)
         (name string)
         (proto simple int)))
*/

int do_init_afunix_source(struct log_handler *c)
{
	CAST(afunix_source, self, c);
	int fd;
	struct sockaddr_un sun;
	
	sun.sun_family = AF_UNIX;
	strncpy(sun.sun_path, self->name->data, sizeof(sun.sun_path));
	if (open_unix_socket(&sun, self->proto, &fd)) {
		switch (self->proto) {
		case AFUNIX_PROTO_STREAM:
			if ((self->listen = io_listen(self->backend, fd, 
					make_afunix_accept_callback(self->backend, &self->connections, self->super.super.next))))
				return ST_OK | ST_GOON;
			break;
		case AFUNIX_PROTO_DGRAM:
			self->src = io_read(self->backend, fd, 
				make_read_line(make_log_reader((struct log_handler *) self->super.super.next)), 
				NULL);
			return ST_OK | ST_GOON;
		}
	}
	return ST_FAIL | ST_QUIT;
}

struct log_source_driver *make_afunix_source(const char *name, int proto, struct io_backend *backend)
{
        NEW(afunix_source, self);

        self->super.super.super.init = do_init_afunix_source;
        self->backend = backend;
        self->name = c_format("%lz%c", name, 0);
        self->name->length--;
        self->proto = proto;
        return &self->super;
}


#if 0


struct afunix_source_connection {
	struct base_object header;
	struct afunix_source_driver *src;
	struct afunix_source_connection *next;
	int fd;
};

static int do_afunix_source_conn_io_callback(int fd, int reason, void *data)
{
	struct afunix_source_connection *self = (struct afunix_source_connection *) data;

	
	io_add_read_callback(self->fd, do_afunix_source_conn_io_callback, self);
	return 0;
}

static void free_afunix_source_connection(struct base_object *c);

struct afunix_source_connection *make_afunix_source_connection(struct afunix_source_driver *src, int fd)
{
	struct afunix_source_connection *self = NEW(afunix_source_connection);
	
	self->header.dispose = free_afunix_source_connection;
	self->src = src;
	self->fd = fd;
	io_add_read_callback(fd, do_afunix_source_conn_io_callback, self);
	return self;
}

static void free_afunix_source_connection(struct base_object *c)
{
	struct afunix_source_connection *self = (struct afunix_source_connection *) c;
	if (self->fd != -1)
		close(self->fd);
}

struct afunix_source_driver {
	struct source_driver super;
	struct afunix_source_connection *conns;
	char *fname;
	int proto;
	int fd;
};

static int do_afunix_source_io_callback(int fd, int reason, void *data)
{
	struct afunix_source_driver *self = (struct afunix_source_driver *) data;
	struct afunix_source_connection *conn;
	char buf[MAX_LOG_LINE + 1];
	struct sockaddr_un sun;
	int sun_len, new_fd;
	
	switch (self->proto) {
	case AFUNIX_PROTO_DGRAM:
		sun_len = sizeof(sun);
		recvfrom(self->fd, buf, MAX_LOG_LINE, 0, &sun, &sun_len);
		break;
	case AFUNIX_PROTO_STREAM:
		sun_len = sizeof(sun);
		new_fd = accept(self->fd, &sun, &sun_len);
		if (new_fd != -1) {
			conn = make_afunix_source_connection(self, new_fd);
			conn->next = self->conns;
			self->conns = conn;
		}
		break;
	}
	return 0;
}

static int afunix_open_socket(struct afunix_source_driver *self)
{
	int fd;
	int proto_type = (self->proto == AFUNIX_PROTO_STREAM ? SOCK_STREAM : SOCK_DGRAM);
	
	fd = socket(AF_UNIX, proto_type, 0);
	if (fd != -1) {
		io_init_fd(fd);
		/* bind */
		io_add_read_callback(fd, do_afunix_source_io_callback, self);
	}
	return 0;
}

static void free_afunix_source_driver(struct base_object *c);

struct source_driver *make_afunix_source_driver(char *fname, int proto)
{
	struct afunix_source_driver *self = NEW(afunix_source_driver);
	
	self->super.header.dispose = free_afunix_source_driver;
	self->super.pipe = NULL;
	self->super.next = NULL;
	self->fname = strdup(fname);
	self->proto = proto;
	self->fd = -1;
	return &self->super;
}

/* destructor */
static void free_afunix_source_driver(struct base_object *c)
{
	struct afunix_source_driver *self = (struct afunix_source_driver *) c;
	struct afunix_source_connection *conn, *conn2;
	
	if (self->fd != -1)
		close(self->fd);
	free(self->fname);
	conn = self->conns;
	conn2 = NULL;
	while (conn) {
		conn2 = conn->next;
		DISPOSE(conn);
		conn = conn2;
	}
}

#endif
