/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: cfgfile.c,v 1.5 1999/03/15 13:12:30 bazsi Exp $
 *
 ***************************************************************************/

#include "cfgfile.h"

#include <stdio.h>
#include <string.h>
#include "cfg-grammar.h"

#define CLASS_DEFINE
#include "cfgfile.h.x"
#undef CLASS_DEFINE

struct syslog_conf *configuration;

void add_dest_group(struct log_dest_group *grp)
{
	grp->next_dest_group = configuration->destinations;
	configuration->destinations = grp;
}

void add_source_group(struct log_source_group *grp)
{
	grp->next_source_group = configuration->sources;
	configuration->sources = grp;
}

void add_log_connection(struct log_connection *conn)
{
	conn->next = configuration->connections;
	configuration->connections = conn;
}

static int do_init_config(struct syslog_conf *self)
{
	struct log_source_group *s;
	struct log_dest_group *d;
	struct log_connection *c;
	struct log_endpoint_info *i;
	struct log_handler *center;
	int res;

	center = make_log_center(self->connections);
	
	for (s = self->sources; s; s = s->next_source_group) {
		res = INIT_LOG_HANDLER(s);
		append_log_handler(s, center);
		if (res & ST_FAIL)
			return 0;
	}
	
	for (d = self->destinations; d; d = d->next_dest_group) {
		res = INIT_LOG_HANDLER(d);
		if (res & ST_FAIL)
			return 0;
	}
	
	for (c = self->connections; c; c = c->next) {
		for (i = c->nodes; i; i = i->next) {
			switch (i->type) {
			case EP_SOURCE:
				for (s = self->sources; s; s = s->next_source_group) {
					if (strncmp(i->name->data, s->name->data, i->name->length) == 0) {
						i->ref = (struct ol_object *) s;
						break;
					} 
				}
				if (i->ref == NULL) {
					wwrite("unresolved source reference\n");
					return 0;
				}
				break;
			case EP_DESTINATION:
				for (d = self->destinations; d; d = d->next_dest_group) {
					if (strncmp(i->name->data, d->name->data, i->name->length) == 0) {
						i->ref = (struct ol_object *) d;
					} 
				}
				if (i->ref == NULL) {
					wwrite("unresolved destination reference\n");
					return 0;
				}
				break;
			}
		}
	}

	return 1;
}

extern FILE *yyin;
extern int yyparse();
extern int yydebug;

struct syslog_conf *make_syslog_conf(const char *name, struct io_backend *backend)
{
	FILE *cfg;
	int res;
	
	NEW(syslog_conf, self);
	
	configuration = self;
	self->init = do_init_config;
	self->backend = backend;
	if ((cfg = fopen(name, "r")) != NULL) {
		yyin = cfg;
		/* yydebug = 1; */
		res = yyparse();
		fclose(cfg);
		if (!res) {
			/* successfully parsed */
			configuration = NULL;
			return self;
		}
	}
	KILL(self);
	configuration = NULL;
	return NULL;
}

