/***************************************************************************
 *
 * Portions copyright (C) 1998-1999 Niels Mller
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: format.h,v 1.1.1.1 1999/02/08 14:47:33 bazsi Exp $
 *
 ***************************************************************************/

#ifndef __FORMAT_H_INCLUDED
#define __FORMAT_H_INCLUDED

#include "objects.h"
#include "lstring.h"

#include <stdarg.h>

/* Format strings can contain the following %-specifications:
 *
 * %%  Insert a %-sign
 *
 * %c  Insert an 8-bit character
 *
 * %i  Insert a 32-bit integer, in network byte order
 *
 * %d  Insert a 32-bit integer, in decimal
 *
 * %s  Insert a string, given by a length and a pointer.
 *
 * %S  Insert a string, given as a struct long_string pointer.
 *
 * %z  Insert a string, using a null-terminated argument.
 *
 * %r  Reserves space in the string, and stores a pointer to this space
 *     into the given UINT8 ** argument.
 *
 * There are three valid modifiers:
 *
 * "l" (as in literal). It is applicable to the s, a, A, n and r
 * specifiers, and outputs strings *without* a length field.
 *
 * "e" (as in expression). Formats the input string using sexp syntax;
 * i.e. prefixed with the length in decimal.
 *
 * "f" (as in free). Frees the input string after it has been copied.
 * Applicable to %S only.
 */

struct long_string *c_format(const char *format, ...);
UINT32 c_format_length(const char *format, ...);
void c_format_write(const char *format, UINT32 length, UINT8 *buffer, ...);

UINT32 c_vformat_length(const char *format, va_list args);
void c_vformat_write(const char *format, UINT32 length, UINT8 *buffer, va_list args);

/* Short cut */
#define long_string_dup(s) (c_format("%lS", (s)))

struct long_string *format_cstring(const char *s);
struct long_string *make_cstring(struct long_string *s, int free);

#endif /* LSH_FORMAT_H_INCLUDED */
