/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: line_buffer.c,v 1.4 1999/03/08 19:25:37 bazsi Exp $
 *
 ***************************************************************************/
 
#include "syslog-ng.h"

#include "line_buffer.h"
#include "lstring.h"
#include "list.h"

#include "line_buffer.c.x"

/* CLASS:
     (class
       (name line_buffer)
       (super abstract_buffer)
       (vars
         (fifo object object_list)
         (first simple int)
         (last simple int)
         (fifo_size simple int)))
*/

#define GET_NEXT_SLOT(ptr, max_slot) ((ptr) == (max_slot - 1) ? 0 : (ptr + 1))

static int do_buf_line(struct abstract_write *c, struct long_string *line)
{
	CAST(line_buffer, self, c);
	int next_slot;
	
	next_slot = GET_NEXT_SLOT(self->last, self->fifo_size);
	if (next_slot == self->first) {
		/* fifo full */
		return ST_FAIL | ST_GOON;
	}
	else {
		LIST(self->fifo)[self->last] = (struct ol_object *) line;
		self->last = next_slot;
	}
	return ST_OK | ST_GOON;
}

static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
{
	CAST(line_buffer, self, c); 
	struct long_string *item; 
	int res;
	
	if (self->first != self->last) {
		item = (struct long_string *) LIST(self->fifo)[self->first];
		res = A_WRITE(w, item);
		if (res > 0) {
			self->first = GET_NEXT_SLOT(self->first, self->fifo_size);
			long_string_free(item);
		}
		return 0;
	}
	return 0;
}

static int do_prepare_write(struct abstract_buffer *c)
{
	CAST(line_buffer, self, c);
	
	return self->first != self->last;
}

struct abstract_buffer *make_line_buffer(int fifo_size)
{
	NEW(line_buffer, self);
	
	self->super.super.write = do_buf_line;
	self->super.flush = do_flush;
	self->super.prepare = do_prepare_write;
	self->fifo = alloc_object_list(fifo_size);
	self->fifo_size = fifo_size;
	
	return &self->super;
}
