/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: sources.c,v 1.7 1999/03/08 19:25:37 bazsi Exp $
 *
 ***************************************************************************/

#include "sources.h"
#include "xalloc.h"
#include "format.h"

#include <stdlib.h>
#include <assert.h>

#define  CLASS_DEFINE
#include "sources.h.x"
#undef CLASS_DEFINE

#include "sources.c.x"

/* CLASS:
     (class
       (name log_reader)
       (super line_handler)
       (vars
         (next object log_handler)))
*/

static int do_handle_line(struct line_handler **c, struct read_handler **r, UINT32 length, UINT8 *data)
{
	CAST(log_reader, self, *c);
	struct log_info *logmsg;

	logmsg = make_log_info(c_format("%ls", length, data));
	HANDLE_LOG(self->next, logmsg);
	return ST_OK | ST_GOON;
}

struct line_handler *make_log_reader(struct log_handler *next)
{
	NEW(log_reader, self);

	self->super.handler = do_handle_line;
	self->next = next;
	
	return &self->super;
}

/* source_group */

static void do_add_source_name(struct log_handler *c, struct log_info *logmsg)
{
	CAST(log_source_group, self, c);
  
	/* TODO: add source name to logmsg */
	assert(self->super.next);
	logmsg->source = c;
	HANDLE_LOG(self->super.next, logmsg);
}

static int do_init_group(struct log_handler *c)
{
	CAST(log_source_group, self, c);
	int res;
	
	struct log_source_driver *drv;
	
	res = 0;
	for (drv = self->drivers; drv; drv = drv->next_driver) {
		res = INIT_LOG_HANDLER(drv);
		if (res & ST_QUIT) 
			break;
	}
	return res;
}

void set_source_drivers(struct log_source_group *src, struct log_source_driver *drvs)
{
	struct log_source_driver *drv;
	
	src->drivers = drvs;
	for (drv = src->drivers; drv; drv = drv->next_driver) {
		append_log_handler(drv, src);
	}
}

struct log_source_group *make_source_group(const char *name, struct log_source_driver *drvs)
{
	NEW(log_source_group, self);
  
	self->super.super.init = do_init_group;  
	self->super.super.handler = do_add_source_name;
	self->name = c_format("%lz", name);
	set_source_drivers(self, drvs);
	
	return self;
}



#if 0
static int do_source_group_send(struct logmsg_pipe *src, struct logmsg *msg)
{
	struct source_group *self = (struct source_group *) src;

	msg->src = self;
	return 0;
}

static void free_source_group(struct base_object *c);

struct source_group *make_source_group(void)
{
	struct source_group *self = NEW(source_group);

	self->super.header.dispose = free_source_group;	
	self->super.send_msg = do_source_group_send;
	
	return self;
}

static void free_source_group(struct base_object *c)
{
	struct source_group *self = (struct source_group *) c;
	struct source_driver *drv, *drv2;
	
	drv2 = NULL;
	drv = self->drivers;
	while (drv) {
		drv2 = drv->next;
		DISPOSE(drv);
		drv = drv2;
	}
}

#endif
